# Continuum for Cypress

This is a sample Continuum 6.4.0 project that tests a couple pages on www.nps.gov for accessibility concerns using [Cypress](https://www.cypress.io/), a JavaScript end-to-end testing framework.

A custom Cypress support file `continuum/cypress.js` is loaded automatically before the end-to-end tests defined in `cypress/e2e/a11y.spec.js` are run.
This is accomplished by importing `continuum/cypress.js` inside `cypress/support/e2e.js`, where `cypress/support/e2e.js` is the default support file defined by the property name of the configuration object passed to `defineConfig` function in `cypress.config.js`.
Note that all Cypress fixtures and plugins are disabled in this sample project; to enable them, remove the `fixturesFolder` and `setupNodeEvents(on, config) {}` attributes in `cypress.config.js`, respectively, to have their values revert to their defaults.
For more information about Cypress' configuration file, refer to [Cypress' documentation](https://docs.cypress.io/guides/references/configuration.html).

## Prerequisites
* Node

## Installation
1. Install dependencies (including Cypress, if necessary):

        npm install

## Usage
1. Run Cypress:

        npm run cypress:open

2. Click on `E2E Testing` button.

3. Choose a browser and click `Start E2E Testing` button.

4. Click the `a11y.cy.js` entry from the `Specs` list in the Cypress application window.

## Integrating With Level Access Platform

### Configuration
In order to integrate with the Level Access Platform you will need to set some properties in the `continuum/continuum.conf.js`.

    "levelAccessPlatform": {
      "orgInstanceUrl": "https://org.instance.domain/",
      "apiKey": "",
      "workspaceId": "",
      "digitalPropertyId": "",
      "scanTagId": ""
    },

#### Properties
* levelAccessPlatform - This property is used to group all the configuration properties related to integrating with the Level Access Platform.
* orgInstanceUrl - This is the base organization URL where the Level Access Platform services are located. This property is required and will be used for all scan sessions.
* apiKey - This is the API key used in requests to the Level Access Platform services. This property is required and will be used for all scan sessions.
* workspaceId - This is the id or the workspace used when opening a scan session in the Level Access Platform. This property is optional and can either be set here to apply to all scan sessions or provided / overridden programatically when opening a new scan session.
* digitalPropertyId - This is the id or the digital property used when opening a scan session in the Level Access Platform. This property is optional and can either be set here to apply to all scan sessions or provided / overridden programatically when opening a new scan session.
* scanTagId - This is the id or the scan tag used when opening a scan session in the Level Access Platform. This property is optional and can either be set here to apply to all scan sessions or provided / overridden programatically when opening a new scan session.

### Test For Assertions
The Level Access Platform accepts test results in the form of assertions. In order for Continuum to produce test results in this format you will need to call the `runAllTestsForAssertions()` method. This will return an array of `Assertion` objects.

### Get The Reporting Service
Submitting results to the Level Access Platform is done through a reporting service. You will need to retrieve an instance of the `LevelAccessPlatformReportingService` from Continuum by accessing the `LevelAccessPlatformReportingService` property. The returned reporting service instance will allow you to open and complete scan sessions and submit test results.

### Open A Scan Session
Before you can submit any test results to the Level Access Platform you will need to have an active scan session. You can open a new scan session by calling the `openScanSession()` method on the `LevelAccessPlatformReportingService` instance retrieved from Continuum. A name will need to be provided for your scan session.

### Submit Results
Once you have opened a scan session you will be able to submit test results to the Level Access Platform. This can be done by calling the `submit()` method on the `LevelAccessPlatformReportingService` instance retrieved from Continuum passing an array of `Assertion` objects. The supplied assertions will be ent to the Level Access Platform and stored as findings for the URL the browser is currently at. The page title and screen dimensions will also be automatically recorded and sent along with the assertions.

### Complete The Scan Session
When you have finished testing you can complete your scan session by calling the `completeScanSession()` method on the `LevelAccessPlatformReportingService` instance retrieved from Continuum.

### Example
1. Update your `cypress/e2e/a11y.cy.js` file to include the following:

       context('Example Tests with Continuum posting Assertions to the Level Access Platform', () => {
           before(() => {
                 // Continuum only needs to be set up once per testing context;
                 // the page under test can change without having to set up Continuum again
                 cy.setUpContinuum('continuum/continuum.conf.js')
           })

           it('Browse nps.gov', () => {
              cy.visit('https://www.nps.gov/')
                  .runAllTestsForAssertions() // this takes an optional boolean argument that determines if the contents of an iframe will be scanned
                  .submitAssertionsToLevelAccessPlatform()
                  .failIfAnyAssertions()
          })
       })

2. Update your `continuum/cypress.js` file to include the following 

       const runAllTestsForAssertions = (includeiframe = false) => (
          // We verify Access Engine is loaded, loading it again only if necessary, before running our accessibility tests using `Continuum.runAllTestsForAssertions`
          cy.window()
              .then(windowUnderTest => (
                  cy.then(() => {
                      if (!windowUnderTest.LevelAccess_Continuum_AccessEngine) {
                          return cy.readFile(accessEngineFilePath)
                              .then(accessEngineFileContents => windowUnderTest.eval(Continuum.createInjectableAccessEngineCode(accessEngineFileContents)));
                      }
                  })
              ))
              .then(() => Continuum.runAllTestsForAssertions(includeiframe))
       );

       const failIfAnyAssertions = () => {
           const failedAssertions = Continuum.getAssertions().filter((assertion) => {
               return assertion.outcome === 'fail';
           })
           expect(failedAssertions, 'no assertions').to.have.lengthOf(0);
       }
   
       const submitAssertionsToLevelAccessPlatform = async () => {
           const levelAccessPlatformReportingService = Continuum.LevelAccessPlatformReportingService;
           await levelAccessPlatformReportingService.openScanSession({
               sessionName: 'JS Cypress'
           });
   
           try {
               const assertions = Continuum.getAssertions();
               await levelAccessPlatformReportingService.submit(assertions);
           } finally {
               await levelAccessPlatformReportingService.completeScanSession();
           }
       }
      
       Cypress.Commands.add('runAllTestsForAssertions', runAllTestsForAssertions);
       Cypress.Commands.add('failIfAnyAssertions', failIfAnyAssertions);
       Cypress.Commands.add('submitAssertionsToLevelAccessPlatform', submitAssertionsToLevelAccessPlatform);

## Support
Rich API documentation for all the functionality supported out of the box with our Continuum JavaScript SDK, which this sample project uses, can be found [here](https://tools.levelaccess.net/continuum/docs/latest/jsdoc/index.html).

Questions can be emailed to Level Access support at support@levelaccess.com.
